// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "SNAP7Hardware.h"

/*----------------------------------------------------------*/
/* Time funtion
 * IeRfcSetTime: Convert lynxos time_t format to PLC _DT format
 * (SIMATIC).
 *
 * Details:
 * PLC DATE_AND_TIME (or DT) format is coded on 8 byte in BCD
 * format:
 * 0      1      2      3      4      5      6       7
 * YY     MM     DD     HH     MN     SC     SC/100  SC/10e4
 */
void IeRfcSetTime(unsigned char *dt, time_t epoch)  // to PLC
{
  struct tm *dscst;
  dscst = localtime(&epoch);

  dt[7] = 0;   // not used at the moment
  dt[6] = 0;   // not used at the moment
  dt[5] = _tobcd(dscst->tm_sec);
  dt[4] = _tobcd(dscst->tm_min);
  dt[3] = _tobcd(dscst->tm_hour);
  dt[2] = _tobcd(dscst->tm_mday);
  dt[1] = _tobcd(dscst->tm_mon+1);
  dt[0] = _tobcd(dscst->tm_year-100);
}

/*----------------------------------------------------------*/
/* Time funtion
 * IeRfcGetTime: Convert PLC _DT format to time_t format (SIMATIC).
 *
 * Details:
 * PLC DATE_AND_TIME (or DT) format is coded on 8 byte in BCD
 * format:
 * 0      1      2      3      4      5      6       7
 * YY     MM     DD     HH     MN     SC     SC/100  SC/10e4
 */
double IeRfcGetTime(unsigned char *dt) // from PLC
{
    struct tm plcst;
    time_t plctm;
    double ms;
    int year;

  plcst.tm_sec  = _frombcd(dt[5]);
  plcst.tm_min  = _frombcd(dt[4]);
  plcst.tm_hour = _frombcd(dt[3]);
  plcst.tm_mday = _frombcd(dt[2]);
  plcst.tm_mon  = _frombcd(dt[1])-1;
  year = _frombcd(dt[0]);
  //look at Siemens DATA_AND_TIME type documentation
  plcst.tm_year = ((year >= 90) ? year : year+100);

  ms = ((double)_frombcd(dt[6])/100.)+((double)_frombcd(dt[7])/10000.);
  plcst.tm_isdst = -1; // daylight saving time unavailable
  plctm = mktime(&plcst);

  return((double)plctm + ms);
}
